Stan Math Library: stan/math/prim/scal/prob/skew_normal_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

skew_normal_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_SKEW_NORMAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_SKEW_NORMAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
5 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
6 #include <stan/math/prim/scal/err/check_finite.hpp>
7 #include <stan/math/prim/scal/err/check_not_nan.hpp>
8 #include <stan/math/prim/scal/err/check_positive.hpp>
9 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
10 #include <stan/math/prim/scal/fun/owens_t.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
13 #include <stan/math/prim/scal/meta/include_summand.hpp>
14 #include <stan/math/prim/scal/fun/constants.hpp>
15 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <boost/math/distributions.hpp>
19 #include <cmath>
20
21 namespace stan {
22
23 namespace math {
24
25 template <bool propto,
26 typename T_y, typename T_loc, typename T_scale, typename T_shape>
27 typename return_type<T_y, T_loc, T_scale, T_shape>::type
28 skew_normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma,
29 const T_shape& alpha) {
30 static const char* function("stan::math::skew_normal_log");
31 typedef typename stan::partials_return_type<T_y, T_loc,
32 T_scale, T_shape>::type
33 T_partials_return;
34
35 using std::log;
36 using stan::is_constant_struct;
37 using stan::math::check_positive;
38 using stan::math::check_finite;
39 using stan::math::check_not_nan;
40 using stan::math::check_consistent_sizes;
41 using stan::math::value_of;
42 using stan::math::include_summand;
43 using std::exp;
44
45 // check if any vectors are zero length
46 if (!(stan::length(y)
47 && stan::length(mu)
48 && stan::length(sigma)
49 && stan::length(alpha)))
50 return 0.0;
51
52 // set up return value accumulator
53 T_partials_return logp(0.0);
54
55 // validate args (here done over var, which should be OK)
56 check_not_nan(function, "Random variable", y);
57 check_finite(function, "Location parameter", mu);
58 check_finite(function, "Shape parameter", alpha);
59 check_positive(function, "Scale parameter", sigma);
60 check_consistent_sizes(function,
61 "Random variable", y,
62 "Location parameter", mu,
63 "Scale parameter", sigma,
64 "Shape paramter", alpha);
65
66 // check if no variables are involved and prop-to
67 if (!include_summand<propto, T_y, T_loc, T_scale, T_shape>::value)
68 return 0.0;
69
70 // set up template expressions wrapping scalars into vector views
71 OperandsAndPartials<T_y, T_loc, T_scale, T_shape>
72 operands_and_partials(y, mu, sigma, alpha);
73
74 using boost::math::erfc;
75 using boost::math::erf;
76 using std::log;
77
78 VectorView<const T_y> y_vec(y);
79 VectorView<const T_loc> mu_vec(mu);
80 VectorView<const T_scale> sigma_vec(sigma);
81 VectorView<const T_shape> alpha_vec(alpha);
82 size_t N = max_size(y, mu, sigma, alpha);
83
84 VectorBuilder<true, T_partials_return, T_scale> inv_sigma(length(sigma));
85 VectorBuilder<include_summand<propto, T_scale>::value,
86 T_partials_return, T_scale> log_sigma(length(sigma));
87 for (size_t i = 0; i < length(sigma); i++) {
88 inv_sigma[i] = 1.0 / value_of(sigma_vec[i]);
89 if (include_summand<propto, T_scale>::value)
90 log_sigma[i] = log(value_of(sigma_vec[i]));
91 }
92
93 for (size_t n = 0; n < N; n++) {
94 // pull out values of arguments
95 const T_partials_return y_dbl = value_of(y_vec[n]);
96 const T_partials_return mu_dbl = value_of(mu_vec[n]);
97 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
98 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
99
100 // reusable subexpression values
101 const T_partials_return y_minus_mu_over_sigma
102 = (y_dbl - mu_dbl) * inv_sigma[n];
103 const double pi_dbl = stan::math::pi();
104
105 // log probability
106 if (include_summand<propto>::value)
107 logp -= 0.5 * log(2.0 * pi_dbl);
108 if (include_summand<propto, T_scale>::value)
109 logp -= log(sigma_dbl);
110 if (include_summand<propto, T_y, T_loc, T_scale>::value)
111 logp -= y_minus_mu_over_sigma * y_minus_mu_over_sigma / 2.0;
112 if (include_summand<propto, T_y, T_loc, T_scale, T_shape>::value)
113 logp += log(erfc(-alpha_dbl * y_minus_mu_over_sigma
114 / std::sqrt(2.0)));
115
116 // gradients
117 T_partials_return deriv_logerf
118 = 2.0 / std::sqrt(pi_dbl)
119 * exp(-alpha_dbl * y_minus_mu_over_sigma / std::sqrt(2.0)
120 * alpha_dbl * y_minus_mu_over_sigma / std::sqrt(2.0))
121 / (1 + erf(alpha_dbl * y_minus_mu_over_sigma
122 / std::sqrt(2.0)));
123 if (!is_constant_struct<T_y>::value)
124 operands_and_partials.d_x1[n]
125 += -y_minus_mu_over_sigma / sigma_dbl
126 + deriv_logerf * alpha_dbl / (sigma_dbl * std::sqrt(2.0));
127 if (!is_constant_struct<T_loc>::value)
128 operands_and_partials.d_x2[n]
129 += y_minus_mu_over_sigma / sigma_dbl
130 + deriv_logerf * -alpha_dbl / (sigma_dbl * std::sqrt(2.0));
131 if (!is_constant_struct<T_scale>::value)
132 operands_and_partials.d_x3[n]
133 += -1.0 / sigma_dbl
134 + y_minus_mu_over_sigma * y_minus_mu_over_sigma / sigma_dbl
135 - deriv_logerf * y_minus_mu_over_sigma * alpha_dbl
136 / (sigma_dbl * std::sqrt(2.0));
137 if (!is_constant_struct<T_shape>::value)
138 operands_and_partials.d_x4[n]
139 += deriv_logerf * y_minus_mu_over_sigma / std::sqrt(2.0);
140 }
141 return operands_and_partials.value(logp);
142 }
143
144 template <typename T_y, typename T_loc, typename T_scale, typename T_shape>
145 inline
146 typename return_type<T_y, T_loc, T_scale, T_shape>::type
147 skew_normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma,
148 const T_shape& alpha) {
149 return skew_normal_log<false>(y, mu, sigma, alpha);
150 }
151 }
152 }
153 #endif
154
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::skew_normal_log
return_type< T_y, T_loc, T_scale, T_shape >::type skew_normal_log(const T_y &y, const T_loc &mu, const T_scale &sigma, const T_shape &alpha)
Definition: skew_normal_log.hpp:28
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::erf
fvar< T > erf(const fvar< T > &x)
Definition: erf.hpp:14
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
owens_t.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.